@@ -1,5 +1,5 @@
 //
-// Copyright 2021 Staysail Systems, Inc. <info@staysail.tech>
+// Copyright 2024 Staysail Systems, Inc. <info@staysail.tech>
 // Copyright 2018 Capitar IT Group BV <info@capitar.com>
 //
 // This software is supplied under the terms of the MIT License, a
@@ -15,7 +15,6 @@ struct nni_aio_expire_q {
 	nni_mtx  eq_mtx;
 	nni_cv   eq_cv;
 	nni_list eq_list;
-	uint32_t eq_len;
 	nni_thr  eq_thr;
 	nni_time eq_next; // next expiration
 	bool     eq_exit;
@@ -38,11 +37,11 @@ static int                nni_aio_expire_q_cnt;
 // free to examine the aio for list membership, etc.  The provider must
 // not call finish more than once though.
 //
-// We use an array of expiration queues, each with it's own lock and
-// condition variable, and expiration thread.  By default this is one
+// We use an array of expiration queues, each with its own lock and
+// condition variable, and expiration thread.  By default, this is one
 // per CPU core present -- the goal being to reduce overall pressure
 // caused by a single lock.  The number of queues (and threads) can
-// be tuned using the NNG_EXPIRE_THREADS tunable.
+// be tuned using the NNG_NUM_EXPIRE_THREADS tunable.
 //
 // We will not permit an AIO
 // to be marked done if an expiration is outstanding.
@@ -89,8 +88,8 @@ static int                nni_aio_expire_q_cnt;
 #define aio_safe_lock(l) nni_mtx_lock(l)
 #define aio_safe_unlock(l) nni_mtx_unlock(l)
 #else
-#define aio_safe_lock(l)
-#define aio_safe_unlock(l)
+#define aio_safe_lock(l) ((void) 1)
+#define aio_safe_unlock(l) ((void) 1)
 #endif
 
 static nni_reap_list aio_reap_list = {
@@ -116,7 +115,7 @@ void
 nni_aio_fini(nni_aio *aio)
 {
 	nni_aio_cancel_fn fn;
-	void	     *arg;
+	void             *arg;
 	nni_aio_expire_q *eq = aio->a_expire_q;
 
 	// This is like aio_close, but we don't want to dispatch
@@ -248,7 +247,21 @@ nni_aio_close(nni_aio *aio)
 void
 nni_aio_set_timeout(nni_aio *aio, nni_duration when)
 {
-	aio->a_timeout = when;
+	aio->a_timeout    = when;
+	aio->a_use_expire = false;
+}
+
+void
+nni_aio_set_expire(nni_aio *aio, nni_time expire)
+{
+	aio->a_expire     = expire;
+	aio->a_use_expire = true;
+}
+
+nng_duration
+nni_aio_get_timeout(nni_aio *aio)
+{
+	return (aio->a_timeout);
 }
 
 void
@@ -315,6 +328,12 @@ nni_aio_wait(nni_aio *aio)
 	nni_task_wait(&aio->a_task);
 }
 
+bool
+nni_aio_busy(nni_aio *aio)
+{
+	return (nni_task_busy(&aio->a_task));
+}
+
 int
 nni_aio_begin(nni_aio *aio)
 {
@@ -331,7 +350,7 @@ nni_aio_begin(nni_aio *aio)
 	NNI_ASSERT(aio->a_cancel_fn == NULL);
 	NNI_ASSERT(!nni_list_node_active(&aio->a_expire_node));
 
-	// Some initialization can be done outside of the lock, because
+	// Some initialization can be done outside the lock, because
 	// we must have exclusive access to the aio.
 	for (unsigned i = 0; i < NNI_NUM_ELEMENTS(aio->a_outputs); i++) {
 		aio->a_outputs[i] = NULL;
@@ -364,7 +383,7 @@ nni_aio_schedule(nni_aio *aio, nni_aio_cancel_fn cancel, void *data)
 {
 	nni_aio_expire_q *eq = aio->a_expire_q;
 
-	if (!aio->a_sleep) {
+	if ((!aio->a_sleep) && (!aio->a_use_expire)) {
 		// Convert the relative timeout to an absolute timeout.
 		switch (aio->a_timeout) {
 		case NNG_DURATION_ZERO:
@@ -406,7 +425,7 @@ void
 nni_aio_abort(nni_aio *aio, int rv)
 {
 	nni_aio_cancel_fn fn;
-	void	     *arg;
+	void             *arg;
 	nni_aio_expire_q *eq = aio->a_expire_q;
 
 	nni_mtx_lock(&eq->eq_mtx);
@@ -442,8 +461,9 @@ nni_aio_finish_impl(
 		aio->a_msg = msg;
 	}
 
-	aio->a_expire = NNI_TIME_NEVER;
-	aio->a_sleep  = false;
+	aio->a_expire     = NNI_TIME_NEVER;
+	aio->a_sleep      = false;
+	aio->a_use_expire = false;
 	nni_mtx_unlock(&eq->eq_mtx);
 
 	if (sync) {
@@ -503,6 +523,40 @@ nni_aio_list_active(nni_aio *aio)
 	return (nni_list_node_active(&aio->a_prov_node));
 }
 
+// completions list.
+// Implementation note: in order to avoid wasting space, we
+// reuse the reap node -- which will be inactive here.
+void
+nni_aio_completions_init(nni_aio_completions *clp)
+{
+	*clp = NULL;
+}
+
+void
+nni_aio_completions_add(
+    nni_aio_completions *clp, nni_aio *aio, int result, size_t count)
+{
+	NNI_ASSERT(!nni_aio_list_active(aio));
+	aio->a_reap_node.rn_next = *clp;
+	aio->a_result            = result;
+	aio->a_count             = count;
+	*clp                     = aio;
+}
+
+void
+nni_aio_completions_run(nni_aio_completions *clp)
+{
+	nni_aio *aio;
+	nni_aio *cl = *clp;
+	*clp        = NULL;
+
+	while ((aio = cl) != NULL) {
+		cl                       = (void *) aio->a_reap_node.rn_next;
+		aio->a_reap_node.rn_next = NULL;
+		nni_aio_finish_sync(aio, aio->a_result, aio->a_count);
+	}
+}
+
 static void
 nni_aio_expire_add(nni_aio *aio)
 {
@@ -546,8 +600,8 @@ nni_aio_expire_loop(void *arg)
 		int      rv;
 		nni_time next;
 
-		next       = q->eq_next;
-		now        = nni_clock();
+		next = q->eq_next;
+		now  = nni_clock();
 
 		// Each time we wake up, we scan the entire list of elements.
 		// We scan forward, moving up to NNI_EXPIRE_Q_SIZE elements
@@ -566,7 +620,7 @@ nni_aio_expire_loop(void *arg)
 			continue;
 		}
 		q->eq_next = NNI_TIME_NEVER;
-		exp_idx = 0;
+		exp_idx    = 0;
 		while (aio != NULL) {
 			if ((aio->a_expire < now) &&
 			    (exp_idx < NNI_EXPIRE_BATCH)) {
@@ -591,7 +645,7 @@ nni_aio_expire_loop(void *arg)
 
 		for (uint32_t i = 0; i < exp_idx; i++) {
 			aio = expires[i];
-			rv = aio->a_expire_ok ? 0 : NNG_ETIMEDOUT;
+			rv  = aio->a_expire_ok ? 0 : NNG_ETIMEDOUT;
 
 			nni_aio_cancel_fn cancel_fn  = aio->a_cancel_fn;
 			void             *cancel_arg = aio->a_cancel_arg;
@@ -619,15 +673,15 @@ nni_aio_expire_loop(void *arg)
 }
 
 void *
-nni_aio_get_prov_extra(nni_aio *aio, unsigned index)
+nni_aio_get_prov_data(nni_aio *aio)
 {
-	return (aio->a_prov_extra[index]);
+	return (aio->a_prov_data);
 }
 
 void
-nni_aio_set_prov_extra(nni_aio *aio, unsigned index, void *data)
+nni_aio_set_prov_data(nni_aio *aio, void *data)
 {
-	aio->a_prov_extra[index] = data;
+	aio->a_prov_data = data;
 }
 
 void
@@ -789,17 +843,29 @@ int
 nni_aio_sys_init(void)
 {
 	int num_thr;
+	int max_thr;
 
-	// We create a thread per CPU core for expiration by default.
-#ifndef NNG_EXPIRE_THREADS
-	num_thr = nni_plat_ncpu();
-#else
-	num_thr = NNG_EXPIRE_THREADS;
+#ifndef NNG_MAX_EXPIRE_THREADS
+#define NNG_MAX_EXPIRE_THREADS 8
 #endif
-	if (num_thr > 256) {
-		num_thr = 256;
-	}
 
+#ifndef NNG_NUM_EXPIRE_THREADS
+#define NNG_NUM_EXPIRE_THREADS (nni_plat_ncpu())
+#endif
+
+	max_thr = (int) nni_init_get_param(
+	    NNG_INIT_MAX_EXPIRE_THREADS, NNG_MAX_EXPIRE_THREADS);
+
+	num_thr = (int) nni_init_get_param(
+	    NNG_INIT_NUM_EXPIRE_THREADS, NNG_NUM_EXPIRE_THREADS);
+
+	if ((max_thr > 0) && (num_thr > max_thr)) {
+		num_thr = max_thr;
+	}
+	if (num_thr < 1) {
+		num_thr = 1;
+	}
+	nni_init_set_effective(NNG_INIT_NUM_EXPIRE_THREADS, num_thr);
 	nni_aio_expire_q_list =
 	    nni_zalloc(sizeof(nni_aio_expire_q *) * num_thr);
 	nni_aio_expire_q_cnt = num_thr;
