@@ -1,5 +1,5 @@
 //
-// Copyright 2020 Staysail Systems, Inc. <info@staysail.tech>
+// Copyright 2021 Staysail Systems, Inc. <info@staysail.tech>
 // Copyright 2018 Capitar IT Group BV <info@capitar.com>
 // Copyright 2018 Devolutions <info@devolutions.net>
 //
@@ -12,8 +12,6 @@
 #ifndef CORE_DIALER_H
 #define CORE_DIALER_H
 
-extern int       nni_dialer_sys_init(void);
-extern void      nni_dialer_sys_fini(void);
 extern int       nni_dialer_find(nni_dialer **, uint32_t);
 extern int       nni_dialer_hold(nni_dialer *);
 extern void      nni_dialer_rele(nni_dialer *);
