@@ -1,5 +1,5 @@
 //
-// Copyright 2020 Staysail Systems, Inc. <info@staysail.tech>
+// Copyright 2023 Staysail Systems, Inc. <info@staysail.tech>
 // Copyright 2018 Capitar IT Group BV <info@capitar.com>
 //
 // This software is supplied under the terms of the MIT License, a
@@ -13,13 +13,18 @@
 #include <string.h>
 
 struct nni_id_entry {
-	uint32_t key;
+	uint64_t key;
 	uint32_t skips;
-	void *   val;
+	void    *val;
 };
 
+static int          id_reg_len = 0;
+static int          id_reg_num = 0;
+static nni_id_map **id_reg_map = NULL;
+static nni_mtx      id_reg_mtx = NNI_MTX_INITIALIZER;
+
 void
-nni_id_map_init(nni_id_map *m, uint32_t lo, uint32_t hi, bool randomize)
+nni_id_map_init(nni_id_map *m, uint64_t lo, uint64_t hi, bool randomize)
 {
 	if (lo == 0) {
 		lo = 1;
@@ -33,15 +38,15 @@ nni_id_map_init(nni_id_map *m, uint32_t lo, uint32_t hi, bool randomize)
 	m->id_count    = 0;
 	m->id_load     = 0;
 	m->id_cap      = 0;
+	m->id_dyn_val  = 0;
 	m->id_max_load = 0;
 	m->id_min_load = 0; // never shrink below this
 	m->id_min_val  = lo;
 	m->id_max_val  = hi;
 	if (randomize) {
-		// NB: The range is inclusive.
-		m->id_dyn_val = nni_random() % ((hi - lo) + 1) + lo;
+		m->id_flags = NNI_ID_FLAG_RANDOM;
 	} else {
-		m->id_dyn_val = lo;
+		m->id_flags = 0;
 	}
 }
 
@@ -63,7 +68,7 @@ nni_id_map_fini(nni_id_map *m)
 #define ID_INDEX(m, j) ((j) & (m->id_cap - 1))
 
 static size_t
-id_find(nni_id_map *m, uint32_t id)
+id_find(nni_id_map *m, uint64_t id)
 {
 	size_t index;
 	size_t start;
@@ -93,7 +98,7 @@ id_find(nni_id_map *m, uint32_t id)
 }
 
 void *
-nni_id_get(nni_id_map *m, uint32_t id)
+nni_id_get(nni_id_map *m, uint64_t id)
 {
 	size_t index;
 	if ((index = id_find(m, id)) == (size_t) -1) {
@@ -102,20 +107,76 @@ nni_id_get(nni_id_map *m, uint32_t id)
 	return (m->id_entries[index].val);
 }
 
+static int
+id_map_register(nni_id_map *m)
+{
+	if ((m->id_flags & (NNI_ID_FLAG_STATIC | NNI_ID_FLAG_REGISTER)) !=
+	    NNI_ID_FLAG_STATIC) {
+		return (0);
+	}
+	nni_mtx_lock(&id_reg_mtx);
+	if (id_reg_len <= id_reg_num) {
+		nni_id_map **mr;
+		int          len = id_reg_len;
+		if (len < 10) {
+			len = 10;
+		} else {
+			len *= 2;
+		}
+		mr = nni_zalloc(sizeof(nni_id_map *) * len);
+		if (mr == NULL) {
+			nni_mtx_unlock(&id_reg_mtx);
+			return (NNG_ENOMEM);
+		}
+		id_reg_len = len;
+		if (id_reg_map != NULL)
+			memcpy(
+			    mr, id_reg_map, id_reg_num * sizeof(nni_id_map *));
+		id_reg_map = mr;
+	}
+	id_reg_map[id_reg_num++] = m;
+	m->id_flags |= NNI_ID_FLAG_REGISTER;
+	nni_mtx_unlock(&id_reg_mtx);
+	return (0);
+}
+
+void
+nni_id_map_sys_fini(void)
+{
+	nni_mtx_lock(&id_reg_mtx);
+	for (int i = 0; i < id_reg_num; i++) {
+		if (id_reg_map[i] != NULL) {
+			nni_id_map_fini(id_reg_map[i]);
+		}
+	}
+	nni_free(id_reg_map, sizeof(nni_id_map *) * id_reg_len);
+	id_reg_map = NULL;
+	id_reg_len = 0;
+	id_reg_num = 0;
+	nni_mtx_unlock(&id_reg_mtx);
+}
+
 static int
 id_resize(nni_id_map *m)
 {
-	size_t        new_cap;
-	size_t        old_cap;
 	nni_id_entry *new_entries;
 	nni_id_entry *old_entries;
+	uint32_t      new_cap;
+	uint32_t      old_cap;
 	uint32_t      i;
+	int           rv;
 
 	if ((m->id_load < m->id_max_load) && (m->id_load >= m->id_min_load)) {
 		// No resize needed.
 		return (0);
 	}
 
+	// if it is a statically declared map, register it so that we
+	// will free it at finalization time
+	if ((rv = id_map_register(m)) != 0) {
+		return (rv);
+	}
+
 	old_cap = m->id_cap;
 	new_cap = 8;
 	while (new_cap < (m->id_count * 2)) {
@@ -173,7 +234,7 @@ id_resize(nni_id_map *m)
 }
 
 int
-nni_id_remove(nni_id_map *m, uint32_t id)
+nni_id_remove(nni_id_map *m, uint64_t id)
 {
 	size_t index;
 	size_t probe;
@@ -191,7 +252,7 @@ nni_id_remove(nni_id_map *m, uint32_t id)
 		nni_id_entry *entry;
 
 		// The load was increased once each hashing operation we used
-		// to place the the item.  Decrement it accordingly.
+		// to place the item.  Decrement it accordingly.
 		m->id_load--;
 		entry = &m->id_entries[probe];
 		if (probe == index) {
@@ -213,7 +274,7 @@ nni_id_remove(nni_id_map *m, uint32_t id)
 }
 
 int
-nni_id_set(nni_id_map *m, uint32_t id, void *val)
+nni_id_set(nni_id_map *m, uint64_t id, void *val)
 {
 	size_t        index;
 	nni_id_entry *ent;
@@ -254,9 +315,9 @@ nni_id_set(nni_id_map *m, uint32_t id, void *val)
 }
 
 int
-nni_id_alloc(nni_id_map *m, uint32_t *idp, void *val)
+nni_id_alloc(nni_id_map *m, uint64_t *idp, void *val)
 {
-	uint32_t id;
+	uint64_t id;
 	int      rv;
 
 	NNI_ASSERT(val != NULL);
@@ -266,6 +327,16 @@ nni_id_alloc(nni_id_map *m, uint32_t *idp, void *val)
 		// Really more like ENOSPC.. the table is filled to max.
 		return (NNG_ENOMEM);
 	}
+	if (m->id_dyn_val == 0) {
+		if (m->id_flags & NNI_ID_FLAG_RANDOM) {
+			// NB: The range is inclusive.
+			m->id_dyn_val = nni_random() %
+			        (m->id_max_val - m->id_min_val + 1) +
+			    m->id_min_val;
+		} else {
+			m->id_dyn_val = m->id_min_val;
+		}
+	}
 
 	for (;;) {
 		id = m->id_dyn_val;
@@ -285,3 +356,14 @@ nni_id_alloc(nni_id_map *m, uint32_t *idp, void *val)
 	}
 	return (rv);
 }
+
+int
+nni_id_alloc32(nni_id_map *m, uint32_t *idp, void *val)
+{
+	uint64_t id;
+	int      rv;
+	rv = nni_id_alloc(m, &id, val);
+	NNI_ASSERT(id < (1ULL << 32));
+	*idp = (uint32_t) id;
+	return (rv);
+}
\ No newline at end of file
