@@ -1,5 +1,5 @@
 //
-// Copyright 2018 Staysail Systems, Inc. <info@staysail.tech>
+// Copyright 2021 Staysail Systems, Inc. <info@staysail.tech>
 // Copyright 2018 Capitar IT Group BV <info@capitar.com>
 //
 // This software is supplied under the terms of the MIT License, a
@@ -30,29 +30,42 @@
 
 struct nni_plat_thr {
 	void (*func)(void *);
-	void * arg;
+	void  *arg;
 	HANDLE handle;
 	DWORD  id;
 };
 
 struct nni_plat_mtx {
 	SRWLOCK srl;
-	DWORD   owner;
-	int     init;
 };
 
+#define NNI_MTX_INITIALIZER  \
+	{                    \
+		SRWLOCK_INIT \
+	}
+
 struct nni_rwlock {
 	SRWLOCK rwl;
 	BOOLEAN exclusive;
 };
 
+#define NNI_RWLOCK_INITIALIZER \
+	{                      \
+		SRWLOCK_INIT   \
+	}
+
 struct nni_plat_cv {
 	CONDITION_VARIABLE cv;
 	PSRWLOCK           srl;
 };
 
+#define NNI_CV_INITIALIZER(mxp)                                    \
+	{                                                          \
+		.srl = (void *) mxp, .cv = CONDITION_VARIABLE_INIT \
+	}
+
 struct nni_atomic_flag {
-	unsigned f;
+	LONG f;
 };
 
 struct nni_atomic_bool {
@@ -67,6 +80,10 @@ struct nni_atomic_u64 {
 	LONGLONG v;
 };
 
+struct nni_atomic_ptr {
+	LONGLONG v;
+};
+
 // nni_win_io is used with io completion ports.  This allows us to get
 // to a specific completion callback without requiring the poller (in the
 // completion port) to know anything about the event itself.
@@ -77,8 +94,8 @@ typedef void (*nni_win_io_cb)(nni_win_io *, int, size_t);
 struct nni_win_io {
 	OVERLAPPED    olpd;
 	HANDLE        f;
-	void *        ptr;
-	nni_aio *     aio;
+	void         *ptr;
+	nni_aio      *aio;
 	nni_win_io_cb cb;
 };
 
